@@ -36,23 +36,17 @@ underlying storage as a slice.
 #![cfg_attr(not(feature = "std"), no_std)]
 #![cfg_attr(all(feature = "alloc", not(feature = "std")), feature(alloc))]
 
-#[cfg(all(feature = "alloc", not(feature = "std")))]
-extern crate alloc;
-
-#[cfg(feature = "std")]
-extern crate core;
-
 #[macro_use]
 mod macros;
 
 mod bits;
-mod endian;
+mod cursor;
 mod slice;
 
 pub use crate::{
 	bits::Bits,
-	endian::{
-		Endian,
+	cursor::{
+		Cursor,
 		BigEndian,
 		LittleEndian,
 	},
